/**
 * \file
 *
 * \brief Low-level initialization functions called upon chip startup.
 *
 * Copyright (c) 2017 Atmel Corporation,
 *                    a wholly owned subsidiary of Microchip Technology Inc.
 *
 * \asf_license_start
 *
 * \page License
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the Licence at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * \asf_license_stop
 *
 */

#include "samd51.h"

/**
 * Initial system clock frequency. The System RC Oscillator (RCSYS) provides
 *  the source for the main clock at chip startup.
 */
#define __SYSTEM_CLOCK    (120000000)

uint32_t SystemCoreClock = __SYSTEM_CLOCK;/*!< System Clock Frequency (Core Clock)*/

#define OSCCTRL_BASE_ADDR    (0x40001000u)
#define OSCCTRL_STATUS       (*(volatile unsigned long*)(OSCCTRL_BASE_ADDR + 0x10uL))
#define OSCCTRL_XOSCCTRL0    (*(volatile unsigned long*)(OSCCTRL_BASE_ADDR + 0x14uL))
#define OSCCTRL_XOSCCTRL1    (*(volatile unsigned long*)(OSCCTRL_BASE_ADDR + 0x18uL))
#define OSCCTRL_DPLLCTRLA0   (*(volatile unsigned char*)(OSCCTRL_BASE_ADDR + 0x30uL))
#define OSCCTRL_DPLLRATIO    (*(volatile unsigned char*)(OSCCTRL_BASE_ADDR + 0x34uL))
#define OSCCTRL_DPLLCTRLB0   (*(volatile unsigned char*)(OSCCTRL_BASE_ADDR + 0x38uL))
#define OSCCTRL_DPLLSTATUS0  (*(volatile unsigned long*)(OSCCTRL_BASE_ADDR + 0x40uL))

#define GCLK_BASE_ADDR      (0x40001C00u)
#define GCLK_GENCTRL0       (*(volatile unsigned long*)(GCLK_BASE_ADDR + 0x20uL))  // Generator 0 control
#define GCLK_GENCTRL1       (*(volatile unsigned long*)(GCLK_BASE_ADDR + 0x24uL))  // Generator 1 control
#define GCLK_PCHCTRL1       (*(volatile unsigned long*)(GCLK_BASE_ADDR + 0x84uL))  // Reference clock for FDPLL0

void SetSysClock(void);

/**
 * Initialize the system
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System and update the SystemCoreClock variable.
 */
void SystemInit(void)
{
  // Keep the default device state after reset
  SystemCoreClock = __SYSTEM_CLOCK;
  SetSysClock();
  return;
}

/**
 * Update SystemCoreClock variable
 *
 * @brief  Updates the SystemCoreClock with current core Clock
 *         retrieved from cpu registers.
 */
void SystemCoreClockUpdate(void)
{
  // Not implemented
  SystemCoreClock = __SYSTEM_CLOCK;
  return;
}

/**
  * Set System clock
  *
  * @brief  Configures the System clock frequency
  * @notes  XOSC1(12Mhz) -> GCLK1(2Mhz) -> PLL0(120Mhz) -> GCLK0(120Mhz) -> MCLK
  */
void SetSysClock(void) {
  OSCCTRL_XOSCCTRL1 = 0
                    | ( 1uL <<  1)           // Oscillator enable
                    | ( 1uL <<  2)           // Crystal oscillator enable
                    | ( 1uL <<  8)           // Low buff gain enable
                    | ( 3uL <<  9)           // Oscillator current reference
                    | ( 4uL << 11)           // Oscillator current multiplier
                    | ( 1uL << 15)           // Automatic loop control enable
                    | ( 1uL << 16)           // Clock failure detector enable
                    | (11uL << 20)           // Start-Up time
                    | ( 3uL << 24)           // Clock failure detection prescaler
                    ;
  while((OSCCTRL_STATUS & 0x2) == 0) {       // Wait until HSE is ready
  }

  GCLK_GENCTRL1  |= (1uL <<  0)              // Generator clock source selection = XOSC1
                 |  (1uL <<  8)              // Generator enable                 = Enabled
                 |  (6uL << 16)              // Division factor                  = 6
                 ;

  GCLK_PCHCTRL1  = (1uL << 0)                // Select generator 1 for PLL0
                 | (1uL << 6)                // Enable peripheral channel
                 ;
  OSCCTRL_DPLLRATIO  = (59uL << 0);          // Multiplier of 60 = (59 + 1 + (LDRFRAC / 32))
  OSCCTRL_DPLLCTRLB0 = 0;                    // Set reference clock to GCLK
  OSCCTRL_DPLLCTRLA0 = (1uL << 1);           // Enable PLL 0
  while((OSCCTRL_DPLLSTATUS0 & 0x3) == 0) {  // Wait until PLL0 is ready
  }

  GCLK_GENCTRL0 |= (7uL << 0);               // Set main clock source to PLL0
}
