/*
** ###################################################################
**     Processors:          MIMXRT1176AVM8A_cm7
**                          MIMXRT1176CVM8A_cm7
**                          MIMXRT1176DVMAA_cm7
**
**     Compilers:           Freescale C/C++ for Embedded ARM
**                          GNU C Compiler
**                          IAR ANSI C/C++ Compiler for ARM
**                          Keil ARM C/C++ Compiler
**                          MCUXpresso Compiler
**
**     Reference manual:    IMXRT1170RM_initial_draft, 02/2018
**     Version:             rev. 0.1, 2018-03-05
**     Build:               b191113
**
**     Abstract:
**         Provides a system configuration function and a global variable that
**         contains the system frequency. It configures the device and initializes
**         the oscillator (PLL) that is part of the microcontroller device.
**
**     Copyright 2016 Freescale Semiconductor, Inc.
**     Copyright 2016-2019 NXP
**     All rights reserved.
**
**     SPDX-License-Identifier: BSD-3-Clause
**
**     http:                 www.nxp.com
**     mail:                 support@nxp.com
**
**     Revisions:
**     - rev. 0.1 (2018-03-05)
**         Initial version.
**
** ###################################################################
*/

/*!
 * @file MIMXRT1176_cm7
 * @version 0.1
 * @date 2018-03-05
 * @brief Device specific configuration file for MIMXRT1176_cm7 (implementation
 *        file)
 *
 * Provides a system configuration function and a global variable that contains
 * the system frequency. It configures the device and initializes the oscillator
 * (PLL) that is part of the microcontroller device.
 */

#include <stdint.h>
#include "fsl_device_registers.h"

#define OSC_BASE_ADDR       (0x40C84000u)
#define OSC_24M_CTRL        (*(volatile unsigned int*)(OSC_BASE_ADDR + 0x20u))

#define PMU_REF_CTRL        (*(volatile unsigned int*)(0x40C84570u))
#define PLL_ARM_CTRL        (*(volatile unsigned int*)(0x40C84200u))  // ARM_PLL control register
#define CLOCK_ROOT0_CTRL    (*(volatile unsigned int*)(0x40CC0000u))  // M7 Clock root control register

void SetSysClock (void);

/* ----------------------------------------------------------------------------
   -- Core clock
   ---------------------------------------------------------------------------- */

uint32_t SystemCoreClock = DEFAULT_SYSTEM_CLOCK;

/* ----------------------------------------------------------------------------
   -- SystemInit()
   ---------------------------------------------------------------------------- */

void SystemInit (void) {
#if ((__FPU_PRESENT == 1) && (__FPU_USED == 1))
  SCB->CPACR |= ((3UL << 10*2) | (3UL << 11*2));    /* set CP10, CP11 Full Access */
#endif /* ((__FPU_PRESENT == 1) && (__FPU_USED == 1)) */

#if defined(__MCUXPRESSO)
    extern uint32_t g_pfnVectors[];  // Vector table defined in startup code
    SCB->VTOR = (uint32_t)g_pfnVectors;
#endif

/* Watchdog disable */

#if (DISABLE_WDOG)
    if ((WDOG1->WCR & WDOG_WCR_WDE_MASK) != 0U)
    {
        WDOG1->WCR &= ~WDOG_WCR_WDE_MASK;
    }
    if ((WDOG2->WCR & WDOG_WCR_WDE_MASK) != 0U)
    {
        WDOG2->WCR &= ~WDOG_WCR_WDE_MASK;
    }
    if ((RTWDOG3->CS & RTWDOG_CS_CMD32EN_MASK) != 0U)
    {
        RTWDOG3->CNT = 0xD928C520U; /* 0xD928C520U is the update key */
    }
    else
    {
        RTWDOG3->CNT = 0xC520U;
        RTWDOG3->CNT = 0xD928U;
    }
    RTWDOG3->TOVAL = 0xFFFF;
    RTWDOG3->CS = (uint32_t) ((RTWDOG3->CS) & ~RTWDOG_CS_EN_MASK) | RTWDOG_CS_UPDATE_MASK;
    if ((RTWDOG4->CS & RTWDOG_CS_CMD32EN_MASK) != 0U)
    {
        RTWDOG4->CNT = 0xD928C520U; /* 0xD928C520U is the update key */
    }
    else
    {
        RTWDOG4->CNT = 0xC520U;
        RTWDOG4->CNT = 0xD928U;
    }
    RTWDOG4->TOVAL = 0xFFFF;
    RTWDOG4->CS = (uint32_t) ((RTWDOG4->CS) & ~RTWDOG_CS_EN_MASK) | RTWDOG_CS_UPDATE_MASK;
#endif /* (DISABLE_WDOG) */

    /* Disable Systick which might be enabled by bootrom */
    if ((SysTick->CTRL & SysTick_CTRL_ENABLE_Msk) != 0U)
    {
        SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
    }

/* Enable instruction and data caches */
#if defined(__ICACHE_PRESENT) && __ICACHE_PRESENT
    if (SCB_CCR_IC_Msk != (SCB_CCR_IC_Msk & SCB->CCR)) {
        SCB_EnableICache();
    }
#endif
#if defined(__DCACHE_PRESENT) && __DCACHE_PRESENT
    if (SCB_CCR_DC_Msk != (SCB_CCR_DC_Msk & SCB->CCR)) {
        SCB_EnableDCache();
    }
#endif

  SystemInitHook();
  SetSysClock();
}

/* ----------------------------------------------------------------------------
   -- SystemCoreClockUpdate()
   ---------------------------------------------------------------------------- */

void SystemCoreClockUpdate (void) {
  SystemCoreClock = 996000000u;
}

/* ----------------------------------------------------------------------------
   -- SystemInitHook()
   ---------------------------------------------------------------------------- */


__attribute__ ((weak)) void SystemInitHook (void) {
  /* Void implementation of the weak function. */
}


/* ----------------------------------------------------------------------------
   -- SetSysClock()
   ---------------------------------------------------------------------------- */
//
//
//
void SetSysClock(void) {
  volatile long i;

  //
  // Check if 24Mhz XTAL is already stable and running
  //
  if ((OSC_24M_CTRL & 0x40000090u) != 0x40000010u) {
    OSC_24M_CTRL |=  (1u << 7);                       // Make sure XTAL is gated
    OSC_24M_CTRL |=  (1u << 4);                       // Enable XTAL
    while ((OSC_24M_CTRL & (1u << 30)) == 0) {
      ;  // Wait for XTAL to be stable
    }
    OSC_24M_CTRL &= ~(1u << 7);                       // Set XTAL to be not gated
  }
  //
  // Check if ARM PLL is already stable, running and not gated
  //
  if ((PLL_ARM_CTRL & 0x20002000u) != 0x20002000u) {
    PMU_REF_CTRL |= (1u << 3);                        // Enable the reference voltage
	//
    // Configure ARM PLL
    //   fout   = fin   * divider / 2.0
    //   996Mhz = 12Mhz *     166 / 2.0
    //
    PLL_ARM_CTRL = (PLL_ARM_CTRL & ~(0xFFu))          // Clear divider
                 | (166 << 0);                        // Set divider to 166
    while ((PLL_ARM_CTRL & (1u << 29)) == 0) {
      ;  // Wait for PLL to be stable
    }
    PLL_ARM_CTRL |= (1u << 21);                       // Power up the PLL
    for (i = 0; i < 250; i++) {
      ;  // Wait at least 50us for the PLL to be powered
    }
    PLL_ARM_CTRL |= (1u << 13);                       // Enable clock output
  }
  //
  // Check if CM7 clock root is configured correctly
  //
  if (CLOCK_ROOT0_CTRL != 0x400) {
    CLOCK_ROOT0_CTRL = 0
                     | (0u <<  0)                     // Set DIV to 0
                     | (4u <<  8)                     // Set MUX to ARM_PLL
                     | (0u << 16)                     // Set MFD to 0
                     | (0u << 20)                     // Set MFN to 0
                     | (0u << 24)                     // Set Clock root to enabled
                     ;
  }
}
