/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.1.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : UART implementation

Additional information:

  Device : STM32F103
  Board  : ST STM3210B Eval

  Unit | UART   | Board connector
  ===============================
  0    | USART1  | UART0 (CN6)

*/

#include "BSP_UART.h"
#include "RTOS.h"        // For OS_INT_Enter()/OS_INT_Leave(). Remove this line and OS_INT_* functions if not using OS.
#include "stm32f10x.h"   // Device specific header file, contains CMSIS defines.

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#define BSP_USART1            (0)
#define BSP_UART_BAUDRATE     (38400)
#define BSP_UART_CLOCK        (SystemCoreClock)

#define USART1_BASE_ADDR      (0x40013800u)
#define USART_SR(addr)        (*(volatile unsigned long*)((addr) + 0x00u))
#define USART_DR(addr)        (*(volatile unsigned long*)((addr) + 0x04u))
#define USART_BRR(addr)       (*(volatile unsigned long*)((addr) + 0x08u))
#define USART_CR1(addr)       (*(volatile unsigned long*)((addr) + 0x0Cu))
#define USART_CR2(addr)       (*(volatile unsigned long*)((addr) + 0x10u))
#define USART_CR3(addr)       (*(volatile unsigned long*)((addr) + 0x14u))

#define GPIOA_CRH             (*(volatile unsigned long*)(0x40010804u))

#define UART_BRR_VALUE        (BSP_UART_CLOCK / BSP_UART_BAUDRATE)

#define RCC_BASE_ADDR         (0x40021000uL)
#define RCC_APB2ENR           (*(volatile unsigned long*)(RCC_BASE_ADDR + 0x18))
#define RCC_USART1EN          (0x00004004uL)

#define US_RXRDY              (0x20u)  // RXNE
#define USART_RX_ERROR_FLAGS  (0x0Fu)  // ORE/NE/FE/PE
#define US_TXEMPTY            (0x80u)  // TXE


/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif

void USART1_IRQHandler(void);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       _UART_IRQHandler()
*
*  Function description
*    Called by UART Rx & Tx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are in interrupt context.
*/
static void _UART_IRQHandler(unsigned int Unit, const unsigned long pUSARTBaseAddr) {
  unsigned char Status;
  unsigned char Data;

  OS_INT_EnterNestable();
  Status = USART_SR(pUSARTBaseAddr);               // Read status register.
  //                                               
  // Handle Rx.                                    
  //                                               
  do {                                             
    if (Status & US_RXRDY) {                       // Data received?
      Data = USART_DR(pUSARTBaseAddr);             
      if ((Status & USART_RX_ERROR_FLAGS) == 0) {  // If no errors
        if (_pfReadCB != NULL) {                   
          _pfReadCB(Unit, Data);                   // Process actual byte
        }                                          
      }                                            
    }                                              
    Status = USART_SR(pUSARTBaseAddr);             // Examine current status
  } while (Status & US_RXRDY);                     
  //                                               
  // Handle Tx.                                    
  //                                               
  if (Status & US_TXEMPTY) {                       
    if (_pfWriteCB != NULL && _pfWriteCB(Unit)) {  // No more characters to send ?
      USART_CR1(pUSARTBaseAddr) &= ~0x40uL;        // Disable further tx interrupts
    }
  }
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       USART1_IRQHandler()
*
*  Function description
*    UART1 Rx & Tx interrupt handler.
*/
void USART1_IRQHandler(void) {
  _UART_IRQHandler(BSP_USART1, USART1_BASE_ADDR);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  //
  // Unused parameters. Hard coded USART settings.
  //
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
  BSP_UART_USE_PARA(NumDataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(NumStopBits);
  //
  // Setup clocks, GPIO ports and NVIC IRQs.
  //
  RCC_APB2ENR    |= RCC_USART1EN;                // Enable GPIO port used for USART and USART clock
  GPIOA_CRH       = (GPIOA_CRH & 0xFFFFF00FuL) | 0x000004B0uL;
  NVIC_SetPriority(USART1_IRQn, (1 << __NVIC_PRIO_BITS) - 2);
  NVIC_EnableIRQ(USART1_IRQn);
  //
  // Initialize USART
  //
  USART_CR2(USART1_BASE_ADDR)    = 0x200;
  USART_CR1(USART1_BASE_ADDR)    = 0x2C;
  USART_CR3(USART1_BASE_ADDR)    = 0x00;
  USART_BRR(USART1_BASE_ADDR)  = UART_BRR_VALUE;
  USART_CR1(USART1_BASE_ADDR)   |= 0x2000;         // Enable uart
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
  NVIC_DisableIRQ(USART1_IRQn);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  BSP_UART_USE_PARA(Unit);
  USART_DR(USART1_BASE_ADDR)   = Data;
  USART_CR1(USART1_BASE_ADDR) |= 0x40;  // Enable tx interrupt
}

/*************************** End of file ****************************/
