/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.1.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : BSP for SEGGER emPower PoE board.
*/

#include "BSP.h"
#include "lpc43xx.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED_REG_OFFSET  (11)
#define LED_MAX         ( 5)

#define LED0            (0 + LED_REG_OFFSET)  // A9   P2_11_GPIO1[11]
#define LED1            (1 + LED_REG_OFFSET)  // B9   P2_12_GPIO1[12]
#define LED2            (2 + LED_REG_OFFSET)  // A10  P2_13_GPIO1[13]
#define LED3            (3 + LED_REG_OFFSET)  // B8   P3_4_GPIO1[14]
#define LED4            (4 + LED_REG_OFFSET)  // B7   P3_5_GPIO1[15]

#define LED0_MASK       (1u << LED0)
#define LED1_MASK       (1u << LED1)
#define LED2_MASK       (1u << LED2)
#define LED3_MASK       (1u << LED3)
#define LED4_MASK       (1u << LED4)
#define LED_MASK_ALL    (LED0_MASK | LED1_MASK | LED2_MASK | LED3_MASK | LED4_MASK)

#define SCU_BASE_ADDR   (0x40086000u)
#define SCU_SFSP1_0     (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x0080))   //  Pin configuration register for pin P1_0
#define SCU_SFSP1_1     (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x0084))   //  Pin configuration register for pin P1_1
#define SCU_SFSP2_11    (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x12Cu))   //  Pin configuration register for pin P2_11
#define SCU_SFSP2_12    (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x130u))   //  Pin configuration register for pin P2_12
#define SCU_SFSP2_13    (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x134u))   //  Pin configuration register for pin P2_13
#define SCU_SFSP3_4     (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x190u))   //  Pin configuration register for pin P3_4
#define SCU_SFSP3_5     (*(volatile unsigned int *)(SCU_BASE_ADDR + 0x194u))   //  Pin configuration register for pin P3_5

#define GPIO_BASE_ADDR (0x400F4000u)
#define GPIO_DIR0      (*(volatile unsigned int *)(GPIO_BASE_ADDR + 0x2000))   //  Pin output register
#define GPIO_SET0      (*(volatile unsigned int *)(GPIO_BASE_ADDR + 0x2200))   //  Pin set register
#define GPIO_CLR0      (*(volatile unsigned int *)(GPIO_BASE_ADDR + 0x2280))   //  Pin clear register
#define GPIO_NOT0      (*(volatile unsigned int *)(GPIO_BASE_ADDR + 0x2300))   //  Pin toggle register

#define GPIO_DIR1       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2004u))  //  Pin output register
#define GPIO_SET1       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2204u))  //  Pin set register
#define GPIO_CLR1       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2284u))  //  Pin clear register
#define GPIO_NOT1       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2304u))  //  Pin toggle register

#define GPIO_DIR5       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2014u))  //  Pin output register
#define GPIO_SET5       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2214u))  //  Pin set register
#define GPIO_CLR5       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2214u))  //  Pin clear register
#define GPIO_NOT5       (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x2314u))  //  Pin toggle register


#define ALT_FUNC0     (0)
#define ALT_FUNC1     (1)
#define ALT_FUNC2     (2)
#define ALT_FUNC3     (3)
#define ALT_FUNC4     (4)
#define ALT_FUNC5     (5)
#define ALT_FUNC6     (6)
#define ALT_FUNC7     (7)

/*********************************************************************
*
*       Static const data
*
**********************************************************************
*/

static const unsigned char _LEDs[] = { LED1, LED3, LED0, LED2, LED4 };

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // LPC43 has almost all branch clocks enabled after reset which drains a lot of power
  // Initially, disable all branch clocks that are not absolutely needed (as the M4 core clock etc.)
  // Modules will enable them, where necessary
  //
  // CFG reg: Disable clock and enable auto-mechanism (makes sure that all pending accesses to the clock are performed before disabling it)
  // STAT reg: Wait until clock has been disabled
  //
  // CLK_M4_BUS_CFG => Leave M4 bus clock enabled
  // CLK_M4_M4CORE_CFG => Leave M4 clock enabled
  // CLK_M4_FLASHA_CFG => Leave clock to flash bank A enabled
  // CLK_M4_CREG_CFG   => Leave enabled as used by FW (used by SCT)
  // CLK_M4_USB0_CFG   => Leave enabled as USB driver relies on this being enabled after reset
  // CLK_USB0_CFG      => Leave enabled as USB driver relies on this being enabled after reset
  // BASE_USB0_CLK     => Leave enabled as USB driver relies on this being enabled after reset
  // CLK_M4_WWDT_CLK   => Leave enabled, so watchdog early interrupt can occur
  //
  LPC_CCU1->CLK_APB3_BUS_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_BUS_STAT & (1 << 0));
  LPC_CCU1->CLK_APB3_I2C1_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_I2C1_STAT & (1 << 0));
  LPC_CCU1->CLK_APB3_DAC_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_DAC_STAT & (1 << 0));
  LPC_CCU1->CLK_APB3_ADC0_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_ADC0_STAT & (1 << 0));
  LPC_CCU1->CLK_APB3_ADC1_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_ADC1_STAT & (1 << 0));
  LPC_CCU1->CLK_APB3_CAN0_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB3_CAN0_STAT & (1 << 0));
  LPC_CCU1->CLK_APB1_BUS_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_APB1_BUS_STAT & (1 << 0));
  LPC_CCU1->CLK_APB1_MOTOCONPWM_CFG = (0 << 0);   do {} while (LPC_CCU1->CLK_APB1_MOTOCONPWM_STAT & (1 << 0));
  LPC_CCU1->CLK_APB1_I2C0_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB1_I2C0_STAT & (1 << 0));
  LPC_CCU1->CLK_APB1_I2S_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_APB1_I2S_STAT & (1 << 0));
  LPC_CCU1->CLK_APB1_CAN1_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_APB1_CAN1_STAT & (1 << 0));
  LPC_CCU1->CLK_SPIFI_CFG           = (0 << 0);   do {} while (LPC_CCU1->CLK_SPIFI_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_SPIFI_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_SPIFI_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_GPIO_CFG         = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_GPIO_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_LCD_CFG          = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_LCD_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_ETHERNET_CFG     = (3 << 0);   do {} while (!(LPC_CCU1->CLK_M4_ETHERNET_STAT & (1 << 1)));
  LPC_CCU1->CLK_M4_EMC_CFG          = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_EMC_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_SDIO_CFG         = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_SDIO_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_DMA_CFG          = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_DMA_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_SCT_CFG          = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_SCT_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_USB1_CFG         = (3 << 0);   do {} while (!(LPC_CCU1->CLK_M4_USB1_STAT & (1 << 1)));
  LPC_CCU1->CLK_M4_USB0_CFG         = (3 << 0);   do {} while (!(LPC_CCU1->CLK_M4_USB0_STAT & (1 << 1)));
  LPC_CCU1->CLK_M4_EMCDIV_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_EMCDIV_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_M0APP_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_M0APP_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_USART0_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_USART0_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_UART1_CFG        = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_UART1_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_SSP0_CFG         = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_SSP0_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_TIMER0_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_TIMER0_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_TIMER1_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_TIMER1_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_RITIMER_CFG      = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_RITIMER_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_USART2_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_USART2_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_USART3_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_USART3_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_TIMER2_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_TIMER2_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_TIMER3_CFG       = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_TIMER3_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_SSP1_CFG         = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_SSP1_STAT & (1 << 0));
  LPC_CCU1->CLK_M4_QEI_CFG          = (0 << 0);   do {} while (LPC_CCU1->CLK_M4_QEI_STAT & (1 << 0));
  LPC_CCU1->CLK_PERIPH_BUS_CFG      = (0 << 0);   do {} while (LPC_CCU1->CLK_PERIPH_BUS_STAT & (1 << 0));
  LPC_CCU1->CLK_PERIPH_CORE_CFG     = (0 << 0);   do {} while (LPC_CCU1->CLK_PERIPH_CORE_STAT & (1 << 0));
  LPC_CCU1->CLK_PERIPH_SGPIO_CFG    = (0 << 0);   do {} while (LPC_CCU1->CLK_PERIPH_SGPIO_STAT & (1 << 0));
  LPC_CCU1->CLK_USB0_CFG            = (3 << 0);   do {} while (!(LPC_CCU1->CLK_USB0_STAT & (1 << 1)));
  LPC_CCU1->CLK_USB1_CFG            = (3 << 0);   do {} while (!(LPC_CCU1->CLK_USB1_STAT & (1 << 1)));
  LPC_CCU1->CLK_SPI_CFG             = (0 << 0);   do {} while (LPC_CCU1->CLK_SPI_STAT & (1 << 0));
  //
  // CCU2 clocks
  //
  LPC_CCU2->CLK_APB2_USART3_CFG     = (0 << 0);   do {} while (LPC_CCU2->CLK_APB2_USART3_STAT & (1 << 0));
  LPC_CCU2->CLK_APB2_USART2_CFG     = (0 << 0);   do {} while (LPC_CCU2->CLK_APB2_USART2_STAT & (1 << 0));
  LPC_CCU2->CLK_APB0_UART1_CFG      = (0 << 0);   do {} while (LPC_CCU2->CLK_APB0_UART1_STAT & (1 << 0));
  LPC_CCU2->CLK_APB0_USART0_CFG     = (0 << 0);   do {} while (LPC_CCU2->CLK_APB0_USART0_STAT & (1 << 0));
  LPC_CCU2->CLK_APB2_SSP1_CFG       = (0 << 0);   do {} while (LPC_CCU2->CLK_APB2_SSP1_STAT & (1 << 0));
  LPC_CCU2->CLK_APB0_SSP0_CFG       = (0 << 0);   do {} while (LPC_CCU2->CLK_APB0_SSP0_STAT & (1 << 0));
  LPC_CCU2->CLK_SDIO_CFG            = (0 << 0);   do {} while (LPC_CCU2->CLK_SDIO_STAT & (1 << 0));
  //
  // LPC43 has almost all clocks enabled after reset which drains a lot of power
  // Disable base clocks we do not need
  // CFG: Power down clock and enable auto-blocking while clock unstable
  //
  // BASE_SAFE_CLK Clock for WWDT cannot be disabled
  // BASE_M4_CLK => Leave M4 base clock enabled
//  LPC_CGU->BASE_PERIPH_CLK   = (1 << 0) | (1 << 11);
//  LPC_CGU->BASE_USB0_CLK     = (1 << 0) | (1 << 11);
//  LPC_CGU->BASE_USB1_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_PERIPH_CLK   |= (1 << 11);
  LPC_CGU->BASE_USB0_CLK     |= (1 << 11);
  LPC_CGU->BASE_USB1_CLK     |= (1 << 11);

  LPC_CGU->BASE_SPIFI_CLK    = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_SPI_CLK      = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_PHY_RX_CLK   = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_PHY_TX_CLK   = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_APB1_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_APB3_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_LCD_CLK      = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_SDIO_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_SSP0_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_SSP1_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_UART0_CLK    = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_UART1_CLK    = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_UART2_CLK    = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_UART3_CLK    = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_OUT_CLK      = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_APLL_CLK     = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_CGU_OUT0_CLK = (1 << 0) | (1 << 11);
  LPC_CGU->BASE_CGU_OUT1_CLK = (1 << 0) | (1 << 11);
  //
  // Start initializing our HW
  //
  LPC_CCU1->CLK_M4_GPIO_CFG = (1 << 1) | (1 << 0);                 // Enable GPIO clock
  while ((LPC_CCU1->CLK_M4_GPIO_STAT & (1 << 0)) == 0);            // Wait until GPIO clock has been enabled
  //
  // Set all GPIOs as input
  //
  LPC_GPIO_PORT->DIR[0] = 0x00000000;
  LPC_GPIO_PORT->DIR[1] = 0x00000000;
  LPC_GPIO_PORT->DIR[2] = 0x00000000;
  LPC_GPIO_PORT->DIR[3] = 0x00000000;
  LPC_GPIO_PORT->DIR[4] = 0x00000000;
  LPC_GPIO_PORT->DIR[5] = 0x00000000;
  //
  // By default for all pin function registers: FUNC0
  // Unfortunately, for some pins, FUNC0 is *not* GPIO, but peripheral function
  // Change these pins...
  //
  LPC_SCU->SFSP2_0  = ALT_FUNC4; // GPIO5[0]
  LPC_SCU->SFSP2_1  = ALT_FUNC4; // GPIO5[1]
  LPC_SCU->SFSP2_2  = ALT_FUNC4; // GPIO5[2]
  LPC_SCU->SFSP2_3  = ALT_FUNC4; // GPIO5[3]
  LPC_SCU->SFSP2_4  = ALT_FUNC4; // GPIO5[4]
  LPC_SCU->SFSP2_5  = ALT_FUNC4; // GPIO5[5]
  LPC_SCU->SFSP2_6  = ALT_FUNC4; // GPIO5[6]
  LPC_SCU->SFSP2_8  = ALT_FUNC4; // GPIO5[7]
  LPC_SCU->SFSP3_1  = ALT_FUNC4; // GPIO5[8]
  LPC_SCU->SFSP3_2  = ALT_FUNC4; // GPIO5[9]
  LPC_SCU->SFSP3_7  = ALT_FUNC4; // GPIO5[10]
  LPC_SCU->SFSP3_8  = ALT_FUNC4; // GPIO5[11]
  LPC_SCU->SFSP4_8  = ALT_FUNC4; // GPIO5[12]
  LPC_SCU->SFSP4_9  = ALT_FUNC4; // GPIO5[13]
  LPC_SCU->SFSP4_10 = ALT_FUNC4; // GPIO5[14]
  LPC_SCU->SFSP6_7  = ALT_FUNC4; // GPIO5[15]
  LPC_SCU->SFSP6_8  = ALT_FUNC4; // GPIO5[16]
  LPC_SCU->SFSP9_4  = ALT_FUNC4; // GPIO5[17]
  LPC_SCU->SFSP9_5  = ALT_FUNC4; // GPIO5[18]
  LPC_SCU->SFSPA_4  = ALT_FUNC4; // GPIO5[19]
  LPC_SCU->SFSPB_0  = ALT_FUNC4; // GPIO5[20]
  LPC_SCU->SFSPB_1  = ALT_FUNC4; // GPIO5[21]
  LPC_SCU->SFSPB_2  = ALT_FUNC4; // GPIO5[22]
  LPC_SCU->SFSPB_3  = ALT_FUNC4; // GPIO5[23]
  LPC_SCU->SFSPB_4  = ALT_FUNC4; // GPIO5[24]
  LPC_SCU->SFSPB_5  = ALT_FUNC4; // GPIO5[25]
  LPC_SCU->SFSPB_6  = ALT_FUNC4; // GPIO5[26]
  //
  // GPIO6 & 7 are not present on the LPC device we use
  //
  //
  // Some pins do not even have a GPIO functionality. These pins are a bit tricky, as we need to configure them to some kind of input, so they do not drive anything
  //
  // P1_19 FUNC0: ENET_TX_CLK  (input)
  LPC_SCU->SFSP3_0  = ALT_FUNC5; // Reserved (hopefully input / Hi-Z)
                                 // SFSP3_3 ALT_FUNC0: Reserved (hopefully input / Hi-Z)
  LPC_SCU->SFSP4_7  = ALT_FUNC1; // GP_CLKIN
                                 // P6_0 ALT_FUNC0: Reserved (hopefully input / Hi-Z)
                                 // P8_8 ALT_FUNC0: Reserved (hopefully input / Hi-Z)
                                 // PA_0 ALT_FUNC0: Reserved (hopefully input / Hi-Z)
                                 // PC_0 ALT_FUNC0: Reserved (hopefully input / Hi-Z)
  LPC_SCU->SFSPF_0  = ALT_FUNC1; // GP_CLKIN
  LPC_SCU->SFSPF_4  = ALT_FUNC1; // GP_CLKIN
  LPC_SCU->SFSCLK_0 = ALT_FUNC7; // ENET_TX_CLK (input)
  LPC_SCU->SFSCLK_1 = ALT_FUNC2; // Reserved (hopefully input / Hi-Z)
  LPC_SCU->SFSCLK_2 = ALT_FUNC2; // Reserved (hopefully input / Hi-Z)
  LPC_SCU->SFSCLK_3 = ALT_FUNC2; // Reserved (hopefully input / Hi-Z)
  SCU_SFSP1_0 = 0x00;
  SCU_SFSP1_1 = 0x00;
  //
  // Setup all ethernet pins to dedicated alternate function.
  //
  LPC_SCU->SFSP2_0 = ALT_FUNC7;
  LPC_SCU->SFSP1_17 = ALT_FUNC3;
  LPC_SCU->SFSP1_18 = ALT_FUNC3;
  LPC_SCU->SFSP1_20 = ALT_FUNC3;
  LPC_SCU->SFSP1_19 = ALT_FUNC0;
  LPC_SCU->SFSP0_1 = ALT_FUNC6;
  LPC_SCU->SFSP1_15 = ALT_FUNC3;
  LPC_SCU->SFSP0_0 = ALT_FUNC2;
  LPC_SCU->SFSP1_16 = ALT_FUNC7;
  LPC_SCU->SFSP1_14 = ALT_FUNC0;
  GPIO_DIR1 |= (1UL << 7);
  GPIO_CLR1  = (1UL << 7);
  //
  // Setup EN5V6(P2_9->GPIO1.10) as GPIO, default low
  // EN_LOAD(P2_8->GPIO5.7) as CTOUT0
  //
  LPC_SCU->SFSP2_8 = ALT_FUNC1;
  LPC_SCU->SFSP2_9 = ALT_FUNC0;
  GPIO_DIR1 |= (1UL << 10);
  GPIO_CLR1  = (1UL << 10);
  //
  // Setup LED pin to GPIO, P2_11->GPIO1.11
  // P2_11->GPIO1.11
  // P2_12->GPIO1.12
  // P2_13->GPIO1.13
  // P3_4 ->GPIO1.14
  // P3_5 ->GPIO1.15
  //
  SCU_SFSP2_11 = 0x00;
  SCU_SFSP2_12 = 0x00;
  SCU_SFSP2_13 = 0x00;
  SCU_SFSP3_4  = 0x00;
  SCU_SFSP3_5  = 0x00;
  GPIO_DIR1   |= LED_MASK_ALL;
  GPIO_SET1    = LED_MASK_ALL;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if ((unsigned int)Index < LED_MAX) {
    GPIO_CLR1 = 1u << _LEDs[Index];
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if ((unsigned int)Index < LED_MAX) {
    GPIO_SET1 = 1u << _LEDs[Index];
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if ((unsigned int)Index < LED_MAX) {
    GPIO_NOT1 = 1u << _LEDs[Index];
  }
}

/****** End Of File *************************************************/
