/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.1.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : BSP_UART.c
Purpose : UART implementation for EFM32G21 on custom hardware with
          USART0 & RS485 transmitter on PE13.
*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_INT_Enter()/OS_INT_Leave() . Remove this line and OS_INT_* functions if not using OS.

#include "EFM32G.h"    // Device specific header file, contains CMSIS defines.

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  unsigned long Clock;
  unsigned long Divider;

  BSP_UART_USE_PARA(Unit);

  //
  // The USARTn_CLKDIV register does not only host a DIVider
  // field but instead this field takes into account a fractional divider
  // that is not mentioned near the vast description of the register and
  // field in the documentation.
  //
  // Correctly choosing/calculating the divider is mentioned in the
  // Silabs knowledge base article that can be found here:
  //   https://www.silabs.com/community/mcu/32-bit/knowledge-base.entry.html/2018/02/13/_ready_for_reviewu-ccXM
  //
  // Excerpt from the knowledbe base article:
  //   The baudrate is defined by the following equation:
  //     baudrate = f_hfperclk / (oversample * (1 + (USARTn_CLKDIV / 256)))
  //
  //   Conversely, the formula for determining the USARTn_CLKDIV value, given a desired baudrate, is:
  //     USARTn_CLKDIV = 256 * ((f_hfperclk / (oversample * baudrate_desired)) - 1)
  //
  //   To apply this value to the USART, it is important to set it directly to the USARTn_CLKDIV register.
  //   This value does not go in the DIV bitfield in this register (which would require shifting).
  //   No shifting is applied when writing this register.
  //
  Clock          = SystemCoreClock;
  Divider        = 256 * ((Clock / (16uL * Baudrate)) - 1);
  USART0->CLKDIV = Divider;
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       USART0_RX_IRQHandler()
*
*  Function description
*    UART Rx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are in interrupt context.
*/
void USART0_RX_IRQHandler(void) {
  unsigned char Data;

  Data = USART0->RXDATA;    // Read data, clears pending interrupt flag as well.
  if (_pfReadCB != NULL) {
    OS_INT_Enter();
    _pfReadCB(0u, Data);    // Tell the callback that we have received a new byte.
    OS_INT_Leave();
  }
}

/*********************************************************************
*
*       USART0_TX_IRQHandler()
*
*  Function description
*    UART Tx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are in interrupt context.
*/
void USART0_TX_IRQHandler(void) {
  int AllBytesSent;

  USART0->IFC = USART_IFC_TXC;           // Clear the pending interrupt.
  if (_pfWriteCB != NULL) {
    OS_INT_Enter();
    AllBytesSent = _pfWriteCB(0u);       // Tell the callback that we are ready to send the next byte.
    if (AllBytesSent != 0) {
      GPIO->P[4].DOUTCLR = (1uL << 13);  // Disable RS485 transmitter.
    }
    OS_INT_Leave();
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  unsigned long v;
           int  ConfigError;

  ConfigError = 0;

  //
  // Enable clocks.
  //
  CMU->HFPERCLKEN0 |= (CMU_HFPERCLKEN0_USART0 |
                       CMU_HFPERCLKEN0_GPIO  );
  //
  // Setup port pins.
  //
  GPIO->P[4].MODEH |= GPIO_P_MODEH_MODE11_INPUT;     // Set US0_RX to input enabled.
  GPIO->P[4].MODEH |= GPIO_P_MODEH_MODE10_PUSHPULL;  // Set US0_TX to push-pull output.
  USART0->ROUTE    |= (USART_ROUTE_RXPEN |           // Enable USART RX pin, LOC0, PE11.
                       USART_ROUTE_TXPEN);           // Enable USART TX pin, LOC0, PE10.



  GPIO->P[4].MODEH   |= GPIO_P_MODEH_MODE13_PUSHPULL;  // Set US0_TX to push-pull output.
  GPIO->P[4].DOUTCLR |= (1uL << 13);                   // Disable RS485 transmitter.
  //
  // Configure baud rate.
  //
  _SetBaudrate(Unit, Baudrate);
  //
  // Configure NumDataBits, Parity and NumStopBits.
  //
  v = 0;
  switch (NumStopBits) {
  case BSP_UART_STOP_BITS_1:
    v |= USART_FRAME_STOPBITS_ONE;
    break;
  default:
    ConfigError = 1;
    break;
  }
  switch (Parity) {
  case BSP_UART_PARITY_NONE:
    v |= USART_FRAME_PARITY_NONE;
    break;
  case BSP_UART_PARITY_EVEN:
    v |= USART_FRAME_PARITY_EVEN;
    break;
  case BSP_UART_PARITY_ODD:
    v |= USART_FRAME_PARITY_ODD;
    break;
  default:
    ConfigError = 1;
    break;
  }
  switch (NumDataBits) {
  case BSP_UART_DATA_BITS_8:
    v |= USART_FRAME_DATABITS_EIGHT;
    break;
  case BSP_UART_DATA_BITS_9:
    v |= USART_FRAME_DATABITS_NINE;
    break;
  default:
    ConfigError = 1;
    break;
  }
  if (ConfigError != 0) {
    for (;;) {
      //
      // Configuration error.
      // Unsupported parameters used.
      //
    }
  }
  USART0->FRAME = v;
  //
  // Setup UART interrupt(s).
  //
  NVIC_SetPriority(USART0_RX_IRQn, (1 << __NVIC_PRIO_BITS) - 2);
  NVIC_SetPriority(USART0_TX_IRQn, (1 << __NVIC_PRIO_BITS) - 2);
  NVIC_EnableIRQ(USART0_RX_IRQn);
  NVIC_EnableIRQ(USART0_TX_IRQn);
  //
  // Configure interrupt sources and enable Tx & Rx.
  // If the UART uses a TxEmpty flag, do not enable TxComplete until actually sending data.
  //
  USART0->IEN = (USART_IEN_RXDATAV |
                 USART_IEN_TXC    );
  USART0->CMD = (USART_CMD_CLEARRX |
                 USART_CMD_CLEARTX |
                 USART_CMD_TXEN    |
                 USART_CMD_RXEN   );
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);

  //
  // Disable Rx & Tx and clean its logic.
  //
  USART0->CMD = (USART_CMD_CLEARRX |
                 USART_CMD_CLEARTX);
  //
  // Disable all UART interrupts and clear pending flags.
  //
  USART0->IEN = 0u;
  USART0->IFC = USART0->IF;
  NVIC_DisableIRQ(USART0_TX_IRQn);
  NVIC_DisableIRQ(USART0_RX_IRQn);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);

  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);

  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  BSP_UART_USE_PARA(Unit);

  do {
    //
    // Wait for any previous data to be shifted out.
    //
  } while ((USART0->STATUS & USART_STATUS_TXBL) == 0u);
  GPIO->P[4].DOUTSET = (1uL << 13);  // Enable RS485 transmitter.
  USART0->TXDATA = Data;
}

/*************************** End of file ****************************/
