/*****************************************************************************
 *                   SEGGER Microcontroller GmbH & Co. KG                    *
 *            Solutions for real time microcontroller applications           *
 *****************************************************************************
 *                                                                           *
 *               (c) 2017 SEGGER Microcontroller GmbH & Co. KG               *
 *                                                                           *
 *           Internet: www.segger.com   Support: support@segger.com          *
 *                                                                           *
 *****************************************************************************/

/*****************************************************************************
 *                         Preprocessor Definitions                          *
 *                         ------------------------                          *
 * NO_FPU_ENABLE                                                             *
 *                                                                           *
 *   If defined, FPU will not be enabled.                                    *
 *                                                                           *
 * NO_STACK_INIT                                                             *
 *                                                                           *
 *   If defined, the stack pointer will not be initialised.                  *
 *                                                                           *
 * NO_SYSTEM_INIT                                                            *
 *                                                                           *
 *   If defined, the SystemInit() function will not be called. By default    *
 *   SystemInit() is called after reset to enable the clocks and memories to *
 *   be initialised prior to any C startup initialisation.                   *
 *                                                                           *
 * NO_VTOR_CONFIG                                                            *
 *                                                                           *
 *   If defined, the vector table offset register will not be configured.    *
 *                                                                           *
 * MEMORY_INIT                                                               *
 *                                                                           *
 *   If defined, the MemoryInit() function will be called. By default        *
 *   MemoryInit() is called after SystemInit() to enable an external memory  *
 *   controller.                                                             *
 *                                                                           *
 * STACK_INIT_VAL                                                            *
 *                                                                           *
 *   If defined, specifies the initial stack pointer value. If undefined,    *
 *   the stack pointer will be initialised to point to the end of the        *
 *   RAM segment.                                                            *
 *                                                                           *
 * VECTORS_IN_RAM                                                            *
 *                                                                           *
 *   If defined, the exception vectors will be copied from Flash to RAM.     *
 *                                                                           *
 *****************************************************************************/

  .syntax unified

  .global Reset_Handler
  .global _vectors

  .section .init, "ax"
  .thumb_func

  .equ VTOR_REG, 0xE000ED08
  .equ FPU_CPACR_REG, 0xE000ED88

#ifndef STACK_INIT_VAL
#define STACK_INIT_VAL __stack_end__
#endif

Reset_Handler:
#ifndef NO_STACK_INIT
  /* Initialise main stack */
  ldr r0, =STACK_INIT_VAL
  movs r1, #0x7
  bics r0, r1
  mov sp, r0
#endif

#ifndef NO_SYSTEM_INIT
  /* Initialise system */
  ldr r0, =SystemInit
  blx r0
#endif

#ifdef MEMORY_INIT
  ldr r0, =MemoryInit
  blx r0
#endif

#ifdef VECTORS_IN_RAM
  /* Copy exception vectors into RAM */
  ldr r0, =__vectors_start__
  ldr r1, =__vectors_end__
  ldr r2, =__vectors_ram_start__
1:
  cmp r0, r1
  beq 2f
  ldr r3, [r0]
  str r3, [r2]
  adds r0, r0, #4
  adds r2, r2, #4
  b 1b
2:
#endif

#ifndef NO_VTOR_CONFIG
  /* Configure vector table offset register */
  ldr r0, =VTOR_REG
#ifdef VECTORS_IN_RAM
  ldr r1, =_vectors_ram
#else
  ldr r1, =_vectors
#endif
  str r1, [r0]
#endif

#if (defined(__ARM_ARCH_FPV4_SP_D16__) || defined(__ARM_ARCH_FPV5_D16__)) && !defined(NO_FPU_ENABLE)
  /* Enable FPU */
  ldr r0, =FPU_CPACR_REG
  ldr r1, [r0]
  orr r1, r1, #(0xF << 20)
  str r1, [r0]
  dsb
  isb
#endif

  /* Jump to program start */
  ldr r0, =_start
  blx r0

/**
**===========================================================================
**  Program - Default_Handler
**  Abstract: This code gets called when the processor receives an
**    unexpected interrupt.
**===========================================================================
*/
    .section    .text.Default_Handler,"ax",%progbits
Default_Handler:
InfiniteLoop:
    b   InfiniteLoop
    .size   Default_Handler, .-Default_Handler

/**
**===========================================================================
**  Reset, Exception, and Interrupt vectors
**===========================================================================
*/

  .section .vectors, "ax"
  .code 16
  .align 0

  .extern __stack_end__

_vectors:
  .word       __stack_end__           /* Top of Stack */
  .word       Reset_Handler           /* Reset Handler */
  .word       NMI_Handler             /* NMI Handler */
  .word       HardFault_Handler       /* Hard Fault Handler */
  .word       MemManage_Handler       /* MPU Fault Handler */
  .word       BusFault_Handler        /* Bus Fault Handler */
  .word       UsageFault_Handler      /* Usage Fault Handler */
  .word       Default_Handler         /* Reserved */
  .word       Default_Handler         /* Reserved */
  .word       Default_Handler         /* Reserved */
  .word       Default_Handler         /* Reserved */
  .word       SVC_Handler             /* SVCall Handler */
  .word       DebugMon_Handler        /* Debug Monitor Handler */
  .word       Default_Handler         /* Reserved */
  .word       PendSV_Handler          /* PendSV Handler */
  .word       SysTick_Handler         /* SysTick Handler */

  /* External interrupts */

  .word       EMU_IRQHandler         /* 0: EMU Interrupt         */
  .word       WDOG0_IRQHandler       /* 1: WDOG0 Interrupt       */
  .word       LDMA_IRQHandler        /* 2: LDMA Interrupt        */
  .word       GPIO_EVEN_IRQHandler   /* 3: GPIO_EVEN Interrupt   */
  .word       SMU_IRQHandler         /* 4: SMU Interrupt         */
  .word       TIMER0_IRQHandler      /* 5: TIMER0 Interrupt      */
  .word       USART0_RX_IRQHandler   /* 6: USART0_RX Interrupt   */
  .word       USART0_TX_IRQHandler   /* 7: USART0_TX Interrupt   */
  .word       ACMP0_IRQHandler       /* 8: ACMP0 Interrupt       */
  .word       ADC0_IRQHandler        /* 9: ADC0 Interrupt        */
  .word       IDAC0_IRQHandler       /* 10: IDAC0 Interrupt      */
  .word       I2C0_IRQHandler        /* 11: I2C0 Interrupt       */
  .word       I2C1_IRQHandler        /* 12: I2C1 Interrupt       */
  .word       GPIO_ODD_IRQHandler    /* 13: GPIO_ODD Interrupt   */
  .word       TIMER1_IRQHandler      /* 14: TIMER1 Interrupt     */
  .word       TIMER2_IRQHandler      /* 15: TIMER2 Interrupt     */
  .word       TIMER3_IRQHandler      /* 16: TIMER3 Interrupt     */
  .word       USART1_RX_IRQHandler   /* 17: USART1_RX Interrupt  */
  .word       USART1_TX_IRQHandler   /* 18: USART1_TX Interrupt  */
  .word       USART2_RX_IRQHandler   /* 19: USART2_RX Interrupt  */
  .word       USART2_TX_IRQHandler   /* 20: USART2_TX Interrupt  */
  .word       UART0_RX_IRQHandler    /* 21: UART0_RX Interrupt   */
  .word       UART0_TX_IRQHandler    /* 22: UART0_TX Interrupt   */
  .word       UART1_RX_IRQHandler    /* 23: UART1_RX Interrupt   */
  .word       UART1_TX_IRQHandler    /* 24: UART1_TX Interrupt   */
  .word       LEUART0_IRQHandler     /* 25: LEUART0 Interrupt    */
  .word       LEUART1_IRQHandler     /* 26: LEUART1 Interrupt    */
  .word       LETIMER0_IRQHandler    /* 27: LETIMER0 Interrupt   */
  .word       PCNT0_IRQHandler       /* 28: PCNT0 Interrupt      */
  .word       PCNT1_IRQHandler       /* 29: PCNT1 Interrupt      */
  .word       PCNT2_IRQHandler       /* 30: PCNT2 Interrupt      */
  .word       RTCC_IRQHandler        /* 31: RTCC Interrupt       */
  .word       CMU_IRQHandler         /* 32: CMU Interrupt        */
  .word       MSC_IRQHandler         /* 33: MSC Interrupt        */
  .word       CRYPTO0_IRQHandler     /* 34: CRYPTO0 Interrupt    */
  .word       CRYOTIMER_IRQHandler   /* 35: CRYOTIMER Interrupt  */
  .word       FPUEH_IRQHandler       /* 36: FPUEH Interrupt      */
  .word       USART3_RX_IRQHandler   /* 37: USART3_RX Interrupt  */
  .word       USART3_TX_IRQHandler   /* 38: USART3_TX Interrupt  */
  .word       USART4_RX_IRQHandler   /* 39: USART4_RX Interrupt  */
  .word       USART4_TX_IRQHandler   /* 40: USART4_TX Interrupt  */
  .word       WTIMER0_IRQHandler     /* 41: WTIMER0 Interrupt    */
  .word       WTIMER1_IRQHandler     /* 42: WTIMER1 Interrupt    */
  .word       WTIMER2_IRQHandler     /* 43: WTIMER2 Interrupt    */
  .word       WTIMER3_IRQHandler     /* 44: WTIMER3 Interrupt    */
  .word       I2C2_IRQHandler        /* 45: I2C2 Interrupt       */
  .word       VDAC0_IRQHandler       /* 46: VDAC0 Interrupt      */
  .word       TIMER4_IRQHandler      /* 47: TIMER4 Interrupt     */
  .word       TIMER5_IRQHandler      /* 48: TIMER5 Interrupt     */
  .word       TIMER6_IRQHandler      /* 49: TIMER6 Interrupt     */
  .word       USART5_RX_IRQHandler   /* 50: USART5_RX Interrupt  */
  .word       USART5_TX_IRQHandler   /* 51: USART5_TX Interrupt  */
  .word       CSEN_IRQHandler        /* 52: CSEN Interrupt       */
  .word       LESENSE_IRQHandler     /* 53: LESENSE Interrupt    */
  .word       EBI_IRQHandler         /* 54: EBI Interrupt        */
  .word       ACMP2_IRQHandler       /* 55: ACMP2 Interrupt      */
  .word       ADC1_IRQHandler        /* 56: ADC1 Interrupt       */
  .word       LCD_IRQHandler         /* 57: LCD Interrupt        */
  .word       SDIO_IRQHandler        /* 58: SDIO Interrupt       */
  .word       ETH_IRQHandler         /* 59: ETH Interrupt        */
  .word       CAN0_IRQHandler        /* 60: CAN0 Interrupt       */
  .word       CAN1_IRQHandler        /* 61: CAN1 Interrupt       */
  .word       USB_IRQHandler         /* 62: USB Interrupt        */
  .word       RTC_IRQHandler         /* 63: RTC Interrupt        */
  .word       WDOG1_IRQHandler       /* 64: WDOG1 Interrupt      */
  .word       LETIMER1_IRQHandler    /* 65: LETIMER1 Interrupt   */
  .word       TRNG0_IRQHandler       /* 66: TRNG0 Interrupt      */
  .word       QSPI0_IRQHandler       /* 67: QSPI0 Interrupt      */
_vectors_end:

#ifdef VECTORS_IN_RAM
  .section .vectors_ram, "ax"
  .align 0
  .global _vectors_ram

_vectors_ram:
  .space _vectors_end - _vectors, 0
#endif


/**
**===========================================================================
** Provide weak aliases for each Exception handler to the Default_Handler.
**===========================================================================
*/
  .macro  def_irq_handler handler_name
  .weak   \handler_name
  .set    \handler_name, Default_Handler
  .endm

  def_irq_handler     NMI_Handler
  def_irq_handler     HardFault_Handler
  def_irq_handler     MemManage_Handler
  def_irq_handler     BusFault_Handler
  def_irq_handler     UsageFault_Handler
  def_irq_handler     SVC_Handler
  def_irq_handler     DebugMon_Handler
  def_irq_handler     PendSV_Handler
  def_irq_handler     SysTick_Handler

  def_irq_handler     EMU_IRQHandler
  def_irq_handler     WDOG0_IRQHandler
  def_irq_handler     LDMA_IRQHandler
  def_irq_handler     GPIO_EVEN_IRQHandler
  def_irq_handler     SMU_IRQHandler
  def_irq_handler     TIMER0_IRQHandler
  def_irq_handler     USART0_RX_IRQHandler
  def_irq_handler     USART0_TX_IRQHandler
  def_irq_handler     ACMP0_IRQHandler
  def_irq_handler     ADC0_IRQHandler
  def_irq_handler     IDAC0_IRQHandler
  def_irq_handler     I2C0_IRQHandler
  def_irq_handler     I2C1_IRQHandler
  def_irq_handler     GPIO_ODD_IRQHandler
  def_irq_handler     TIMER1_IRQHandler
  def_irq_handler     TIMER2_IRQHandler
  def_irq_handler     TIMER3_IRQHandler
  def_irq_handler     USART1_RX_IRQHandler
  def_irq_handler     USART1_TX_IRQHandler
  def_irq_handler     USART2_RX_IRQHandler
  def_irq_handler     USART2_TX_IRQHandler
  def_irq_handler     UART0_RX_IRQHandler
  def_irq_handler     UART0_TX_IRQHandler
  def_irq_handler     UART1_RX_IRQHandler
  def_irq_handler     UART1_TX_IRQHandler
  def_irq_handler     LEUART0_IRQHandler
  def_irq_handler     LEUART1_IRQHandler
  def_irq_handler     LETIMER0_IRQHandler
  def_irq_handler     PCNT0_IRQHandler
  def_irq_handler     PCNT1_IRQHandler
  def_irq_handler     PCNT2_IRQHandler
  def_irq_handler     RTCC_IRQHandler
  def_irq_handler     CMU_IRQHandler
  def_irq_handler     MSC_IRQHandler
  def_irq_handler     CRYPTO0_IRQHandler
  def_irq_handler     CRYOTIMER_IRQHandler
  def_irq_handler     FPUEH_IRQHandler
  def_irq_handler     USART3_RX_IRQHandler
  def_irq_handler     USART3_TX_IRQHandler
  def_irq_handler     USART4_RX_IRQHandler
  def_irq_handler     USART4_TX_IRQHandler
  def_irq_handler     WTIMER0_IRQHandler
  def_irq_handler     WTIMER1_IRQHandler
  def_irq_handler     WTIMER2_IRQHandler
  def_irq_handler     WTIMER3_IRQHandler
  def_irq_handler     I2C2_IRQHandler
  def_irq_handler     VDAC0_IRQHandler
  def_irq_handler     TIMER4_IRQHandler
  def_irq_handler     TIMER5_IRQHandler
  def_irq_handler     TIMER6_IRQHandler
  def_irq_handler     USART5_RX_IRQHandler
  def_irq_handler     USART5_TX_IRQHandler
  def_irq_handler     CSEN_IRQHandler
  def_irq_handler     LESENSE_IRQHandler
  def_irq_handler     EBI_IRQHandler
  def_irq_handler     ACMP2_IRQHandler
  def_irq_handler     ADC1_IRQHandler
  def_irq_handler     LCD_IRQHandler
  def_irq_handler     SDIO_IRQHandler
  def_irq_handler     ETH_IRQHandler
  def_irq_handler     CAN0_IRQHandler
  def_irq_handler     CAN1_IRQHandler
  def_irq_handler     USB_IRQHandler
  def_irq_handler     RTC_IRQHandler
  def_irq_handler     WDOG1_IRQHandler
  def_irq_handler     LETIMER1_IRQHandler
  def_irq_handler     TRNG0_IRQHandler
  def_irq_handler     QSPI0_IRQHandler
.end
