/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.1.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for TI Hercules TMS470M MCU Development Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define HET_BASE_ADDR ((unsigned int)0xFFF7B800u)
#define HET_GCR       (*(volatile unsigned int*)(HET_BASE_ADDR + 0x00u))  // Global control register
#define HET_DIR       (*(volatile unsigned int*)(HET_BASE_ADDR + 0x34u))  // Direction register
#define HET_DOUT      (*(volatile unsigned int*)(HET_BASE_ADDR + 0x3Cu))  // Data output register
#define HET_DSET      (*(volatile unsigned int*)(HET_BASE_ADDR + 0x40u))  // Data output set register
#define HET_DCLR      (*(volatile unsigned int*)(HET_BASE_ADDR + 0x44u))  // Data output clear register
#define HET_PULDIS    (*(volatile unsigned int*)(HET_BASE_ADDR + 0x4Cu))  // Pull disable register

#define LED0_BIT      (0)
#define LED1_BIT      (1)
#define LED0_MASK     (1u << LED0_BIT)
#define LED1_MASK     (1u << LED1_BIT)
#define LED_MASK_ALL  (LED0_MASK | LED1_MASK)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*
* Function description
*   Initializes port for LEDs
*/
void BSP_Init(void) {
  HET_GCR    = (1u << 24)         // Enable power-down mode
             | (0u << 16)         // Configure HET as slave
             | (0u <<  8)         // 32 bit RAM access
             | (1u <<  2)         // Clear debug status
             | (1u <<  1)         // Enable suspension upon break
             | (1u <<  0)         // Turn On HET module
             ;
  HET_PULDIS &= ~(LED_MASK_ALL);  // Pull enable
  HET_DIR    |= LED_MASK_ALL;     // Configure to output
  HET_DOUT   &= ~(LED_MASK_ALL);  // Initially clear LEDs
}

/*********************************************************************
*
*       BSP_SetLED()
*
* Function description
*   Specified LED is switched on
*
* Parameters
*   Index: LED's index (0-1)
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    HET_DSET = LED0_MASK;  // Switch on LED0
  } else if (Index == 1) {
    HET_DSET = LED1_MASK;  // Switch on LED1
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*
* Function description
*   Specified LED is switched off
*
* Parameters
*   Index: LED's index (0-1)
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    HET_DCLR = LED0_MASK;  // Switch off LED0
  } else if (Index == 1) {
    HET_DCLR = LED1_MASK;  // Switch off LED1
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*
* Function description
*   Decides wether specified LED is turned on and thus
*   has to be switched off - or vice versa
*
* Parameters
*   Index: LED's index (0-1)
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((HET_DOUT & LED0_MASK) == 0) {  // LED0 is off
      HET_DSET = LED0_MASK;             // Switch on LED0
    } else {                            // LED0 is on
      HET_DCLR = LED0_MASK;             // Switch off LED0
    }
  } else if (Index == 1) {
    if ((HET_DOUT & LED1_MASK) == 0) {  // LED1 is off
      HET_DSET = LED1_MASK;             // Switch on LED1
    } else {                            // LED1 is on
      HET_DCLR = LED1_MASK;             // Switch off LED1
    }
  }
}

/****** End Of File *************************************************/
